@@ -20,8 +20,11 @@ def on_start(connection_manager):
         connection_manager.timeout_in_sec,
     )
     if not res.get("success", True):
+        # Update config time even in failure :
+        connection_manager.conf.last_updated_at = get_unixtime_ms()
         logger.error("Failed to communicate with Aikido Server : %s", res["error"])
     else:
         connection_manager.update_service_config(res)
+        connection_manager.update_blocked_ip_addresses()
         logger.info("Established connection with Aikido Server")
     return res
