@@ -9,18 +9,18 @@ def update_service_config(connection_manager, res):
     """
     Update configuration based on the server's response
     """
-    if res["success"] is False:
+    if res.get("success", False) is False:
         logger.debug(res)
         return
     if "block" in res.keys() and res["block"] != connection_manager.block:
         logger.debug("Updating blocking, setting blocking to : %s", res["block"])
         connection_manager.block = bool(res["block"])
 
-    if res.get("endpoints") is not None:
-        connection_manager.conf = ServiceConfig(
-            endpoints=res.get("endpoints", []),
-            last_updated_at=res.get("configUpdatedAt", get_unixtime_ms()),
-            blocked_uids=res.get("blockedUserIds", []),
-            bypassed_ips=res.get("allowedIPAddresses", []),
-            received_any_stats=res.get("receivedAnyStats", True),
-        )
+    connection_manager.conf = ServiceConfig(
+        endpoints=res.get("endpoints", []),
+        last_updated_at=res.get("configUpdatedAt", get_unixtime_ms()),
+        blocked_uids=res.get("blockedUserIds", []),
+        bypassed_ips=res.get("allowedIPAddresses", []),
+        received_any_stats=res.get("receivedAnyStats", True),
+        blocked_ips=res.get("blockedIPAddresses", []),
+    )
