@@ -1,4 +1,5 @@
-from aikido_zen.background_process.routes import route_to_key, Routes
+from aikido_zen.background_process.routes import Routes
+from aikido_zen.api_discovery.get_api_info import get_api_info
 
 
 class Context:
@@ -23,33 +24,8 @@ class Context:
         self.cookies = cookies
 
 
-# route_to_key tests:
-def test_route_to_key_get():
-    assert route_to_key("GET", "/api/resource") == "GET:/api/resource"
-
-
-def test_route_to_key_post():
-    assert route_to_key("POST", "/api/resource") == "POST:/api/resource"
-
-
-def test_route_to_key_put():
-    assert route_to_key("PUT", "/api/resource") == "PUT:/api/resource"
-
-
-def test_route_to_key_delete():
-    assert route_to_key("DELETE", "/api/resource") == "DELETE:/api/resource"
-
-
-def test_route_to_key_with_query_params():
-    assert route_to_key("GET", "/api/resource?query=1") == "GET:/api/resource?query=1"
-
-
-def test_route_to_key_with_trailing_slash():
-    assert route_to_key("GET", "/api/resource/") == "GET:/api/resource/"
-
-
-def test_route_to_key_empty_path():
-    assert route_to_key("GET", "") == "GET:"
+def gen_route_metadata(method="GET", route="/test", url="http://localhost:5000"):
+    return {"method": method, "route": route, "url": url}
 
 
 def test_initialization():
@@ -58,41 +34,96 @@ def test_initialization():
     assert routes.routes == {}
 
 
-def test_add_route_new():
+def test_get_route():
     routes = Routes(max_size=3)
-    context1 = Context("GET", "/api/resource")
-    routes.add_route(context1)
+
+    routes.initialize_route(gen_route_metadata(method="GET", route="/api/resource1"))
+    routes.increment_route(gen_route_metadata(method="GET", route="/api/resource1"))
+
+    routes.initialize_route(gen_route_metadata(method="POST", route="/api/resource2"))
+    routes.increment_route(gen_route_metadata(method="POST", route="/api/resource2"))
+
+    routes.initialize_route(gen_route_metadata(method="PUT", route="/api/resource3"))
+    routes.increment_route(gen_route_metadata(method="PUT", route="/api/resource3"))
+
+    assert routes.get(gen_route_metadata(method="GET", route="/api/resource1")) == {
+        "method": "GET",
+        "path": "/api/resource1",
+        "hits": 1,
+        "hits_delta_since_sync": 1,
+        "apispec": {},
+    }
+    assert routes.get(gen_route_metadata(method="POST", route="/api/resource2")) == {
+        "method": "POST",
+        "path": "/api/resource2",
+        "hits": 1,
+        "hits_delta_since_sync": 1,
+        "apispec": {},
+    }
+    assert routes.get(gen_route_metadata(method="PUT", route="/api/resource3")) == {
+        "method": "PUT",
+        "path": "/api/resource3",
+        "hits": 1,
+        "hits_delta_since_sync": 1,
+        "apispec": {},
+    }
+    assert routes.get(gen_route_metadata(method="GE", route="/api/resource1")) == None
+    assert routes.get(gen_route_metadata(method="GET", route="/api/resource")) == None
+
+
+def test_initialize_route():
+    routes = Routes(max_size=3)
+    routes.initialize_route(gen_route_metadata(route="/api/resource"))
+    assert len(routes.routes) == 1
+    assert routes.routes["GET:/api/resource"]["hits"] == 0
+
+    # Make sure does not do anything if route does not exist
+    routes.initialize_route(gen_route_metadata(route="/api/resource"))
+    assert len(routes.routes) == 1
+    assert routes.routes["GET:/api/resource"]["hits"] == 0
+
+
+def test_increment_route():
+    routes = Routes(max_size=3)
+
+    routes.initialize_route(gen_route_metadata(route="/api/resource"))
+    routes.increment_route(gen_route_metadata(route="/api/resource"))
     assert len(routes.routes) == 1
     assert routes.routes["GET:/api/resource"]["hits"] == 1
 
 
-def test_add_route_existing():
+def test_increment_route_twice():
     routes = Routes(max_size=3)
-    context1 = Context("GET", "/api/resource")
 
-    routes.add_route(context1)
-    routes.add_route(context1)
+    routes.initialize_route(gen_route_metadata(route="/api/resource"))
+    routes.increment_route(gen_route_metadata(route="/api/resource"))
+    routes.increment_route(gen_route_metadata(route="/api/resource"))
     assert len(routes.routes) == 1
     assert routes.routes["GET:/api/resource"]["hits"] == 2
 
 
+def test_increment_route_that_does_not_exist():
+    routes = Routes(max_size=3)
+    routes.increment_route(gen_route_metadata(route="/api/resource"))
+    routes.increment_route(gen_route_metadata(route="/api/resource"))
+    assert len(routes.routes) == 0
+
+
 def test_clear_routes():
     routes = Routes(max_size=3)
-    context1 = Context("GET", "/api/resource")
-    routes.add_route(context1)
+    routes.initialize_route(gen_route_metadata(route="/api/resource"))
     routes.clear()
     assert len(routes.routes) == 0
 
 
 def test_manage_routes_size_eviction():
     routes = Routes(max_size=2)
-    context1 = Context("GET", "/api/resource1")
-    context2 = Context("GET", "/api/resource2")
-    context3 = Context("GET", "/api/resource3")
 
-    routes.add_route(context1)
-    routes.add_route(context2)
-    routes.add_route(context3)  # This should evict the least used route
+    routes.initialize_route(gen_route_metadata(route="/api/resource1"))
+    routes.initialize_route(gen_route_metadata(route="/api/resource2"))
+    routes.initialize_route(
+        gen_route_metadata(route="/api/resource3")
+    )  # This should evict the least used route
 
     assert len(routes.routes) == 2
     assert (
@@ -102,32 +133,32 @@ def test_manage_routes_size_eviction():
 
 def test_iterable():
     routes = Routes(max_size=3)
-    context1 = Context("GET", "/api/resource1")
-    context2 = Context("POST", "/api/resource2")
-    context3 = Context("PUT", "/api/resource3")
 
-    routes.add_route(context1)
-    routes.add_route(context2)
-    routes.add_route(context3)
+    routes.initialize_route(gen_route_metadata(method="GET", route="/api/resource1"))
+    routes.initialize_route(gen_route_metadata(method="POST", route="/api/resource2"))
+    routes.initialize_route(gen_route_metadata(method="PUT", route="/api/resource3"))
 
     routes_list = list(routes)
     assert len(routes_list) == 3
     assert {
         "method": "GET",
         "path": "/api/resource1",
-        "hits": 1,
+        "hits": 0,
+        "hits_delta_since_sync": 0,
         "apispec": {},
     } in routes_list
     assert {
         "method": "POST",
         "path": "/api/resource2",
-        "hits": 1,
+        "hits": 0,
+        "hits_delta_since_sync": 0,
         "apispec": {},
     } in routes_list
     assert {
         "method": "PUT",
         "path": "/api/resource3",
-        "hits": 1,
+        "hits": 0,
+        "hits_delta_since_sync": 0,
         "apispec": {},
     } in routes_list
 
@@ -136,19 +167,16 @@ def test_len():
     routes = Routes(max_size=3)
     assert len(routes) == 0
 
-    context = Context("GET", "/api/resource")
-    routes.add_route(context)
+    routes.initialize_route(gen_route_metadata(route="/api/resource"))
     assert len(routes) == 1
 
-    context2 = Context("POST", "/api/resource2")
-    routes.add_route(context2)
+    routes.initialize_route(gen_route_metadata(method="POST", route="/api/resource2"))
     assert len(routes) == 2
 
 
 def test_api_discovery_for_new_routes(monkeypatch):
     routes = Routes(max_size=3)
     monkeypatch.setenv("AIKIDO_FEATURE_COLLECT_API_SCHEMA", "1")
-
     context1 = Context(
         "GET",
         "/api/resource1",
@@ -160,54 +188,51 @@ def test_api_discovery_for_new_routes(monkeypatch):
             },
         },
     )
-    routes.add_route(context1)
+    routes.initialize_route(gen_route_metadata(route="/api/resource1"))
 
     routes_list = list(routes)
     assert len(routes_list) == 1
     assert {
         "method": "GET",
         "path": "/api/resource1",
-        "hits": 1,
-        "apispec": {
-            "body": {
-                "schema": {
-                    "properties": {
-                        "user": {
-                            "properties": {
-                                "email": {
-                                    "type": "string",
-                                },
-                                "name": {
-                                    "type": "string",
-                                },
-                                "phone": {"type": "number"},
-                            },
-                            "type": "object",
-                        },
-                    },
-                    "type": "object",
-                },
-                "type": "form-urlencoded",
-            },
-            "auth": None,
-            "query": None,
-        },
+        "hits": 0,
+        "hits_delta_since_sync": 0,
+        "apispec": {},
+    } in routes_list
+
+    apispec = get_api_info(context1)
+    routes.update_route_with_apispec(
+        gen_route_metadata(route="/api/resource1"), apispec
+    )
+
+    routes_list = list(routes)
+    assert len(routes_list) == 1
+    assert {
+        "method": "GET",
+        "path": "/api/resource1",
+        "hits": 0,
+        "hits_delta_since_sync": 0,
+        "apispec": apispec,
     } in routes_list
 
 
 def test_api_discovery_existing_route_empty(monkeypatch):
     routes = Routes(max_size=3)
     monkeypatch.setenv("AIKIDO_FEATURE_COLLECT_API_SCHEMA", "1")
-    context1 = Context("GET", "/api/resource1")
-    routes.add_route(context1)
-    routes_list = list(routes)
+    route_metadata = gen_route_metadata(route="/api/resource1")
 
+    context1 = Context("GET", "/api/resource1")
+    routes.initialize_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context1))
+    routes_list = list(routes)
     assert {
         "method": "GET",
         "path": "/api/resource1",
-        "hits": 1,
+        "hits": 0,
+        "hits_delta_since_sync": 0,
         "apispec": {},
     } in routes_list
+
     context2 = Context(
         "GET",
         "/api/resource1",
@@ -219,14 +244,16 @@ def test_api_discovery_existing_route_empty(monkeypatch):
             },
         },
     )
-    routes.add_route(context2)
+    routes.increment_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context2))
 
     routes_list = list(routes)
     assert len(routes_list) == 1
     assert {
         "method": "GET",
         "path": "/api/resource1",
-        "hits": 2,
+        "hits": 1,
+        "hits_delta_since_sync": 1,
         "apispec": {
             "body": {
                 "schema": {
@@ -257,6 +284,8 @@ def test_api_discovery_existing_route_empty(monkeypatch):
 def test_api_discovery_merge_routes(monkeypatch):
     routes = Routes(max_size=3)
     monkeypatch.setenv("AIKIDO_FEATURE_COLLECT_API_SCHEMA", "1")
+    route_metadata = gen_route_metadata(route="/api/resource1")
+
     context1 = Context(
         "GET",
         "/api/resource1",
@@ -274,8 +303,11 @@ def test_api_discovery_merge_routes(monkeypatch):
         },
         content_type="application/json",
     )
-    routes.add_route(context1)
-    routes.add_route(context2)
+    routes.initialize_route(route_metadata)
+    routes.increment_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context1))
+    routes.increment_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context2))
 
     routes_list = list(routes)
     assert len(routes_list) == 1
@@ -283,6 +315,7 @@ def test_api_discovery_merge_routes(monkeypatch):
         "method": "GET",
         "path": "/api/resource1",
         "hits": 2,
+        "hits_delta_since_sync": 2,
         "apispec": {
             "body": {
                 "schema": {
@@ -317,28 +350,32 @@ def test_merge_body_schema(monkeypatch):
     routes = Routes(200)
     assert list(routes) == []
 
-    routes.add_route(Context("POST", "/body"))
+    routes.initialize_route(gen_route_metadata(method="POST", route="/body"))
+    routes.increment_route(gen_route_metadata(method="POST", route="/body"))
     assert list(routes) == [
         {
             "method": "POST",
             "path": "/body",
             "hits": 1,
+            "hits_delta_since_sync": 1,
             "apispec": {},
         },
     ]
 
-    routes.add_route(
-        Context(
-            "POST",
-            "/body",
-            {"test": "abc", "arr": [1, 2, 3], "sub": {"y": 123}},
-        )
+    context1 = Context(
+        "POST",
+        "/body",
+        {"test": "abc", "arr": [1, 2, 3], "sub": {"y": 123}},
     )
+    route_metadata1 = gen_route_metadata(method="POST", route="/body")
+    routes.increment_route(route_metadata1)
+    routes.update_route_with_apispec(route_metadata1, get_api_info(context1))
     assert list(routes) == [
         {
             "method": "POST",
             "path": "/body",
             "hits": 2,
+            "hits_delta_since_sync": 2,
             "apispec": {
                 "body": {
                     "type": "form-urlencoded",
@@ -364,19 +401,21 @@ def test_merge_body_schema(monkeypatch):
             },
         },
     ]
-    routes.add_route(
-        Context(
-            "POST",
-            "/body",
-            {"test": "abc", "arr": [1, 2, 3], "test2": 1, "sub": {"x": 123}},
-        )
+
+    context2 = Context(
+        "POST",
+        "/body",
+        {"test": "abc", "arr": [1, 2, 3], "test2": 1, "sub": {"x": 123}},
     )
-    routes.add_route(Context("POST", "/body"))
+    routes.increment_route(route_metadata1)
+    routes.update_route_with_apispec(route_metadata1, get_api_info(context2))
+    routes.increment_route(route_metadata1)
     assert list(routes) == [
         {
             "method": "POST",
             "path": "/body",
             "hits": 4,
+            "hits_delta_since_sync": 4,
             "apispec": {
                 "body": {
                     "type": "form-urlencoded",
@@ -410,13 +449,18 @@ def test_add_query_schema(monkeypatch):
     monkeypatch.setenv("AIKIDO_FEATURE_COLLECT_API_SCHEMA", "1")
 
     routes = Routes(200)
+    context = Context("GET", "/query", None, query={"test": "abc", "t": "123"})
+    route_metadata = gen_route_metadata(route="/query")
+
+    routes.initialize_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context))
 
-    routes.add_route(Context("GET", "/query", None, query={"test": "abc", "t": "123"}))
     assert list(routes) == [
         {
             "method": "GET",
             "path": "/query",
-            "hits": 1,
+            "hits": 0,
+            "hits_delta_since_sync": 0,
             "apispec": {
                 "body": None,
                 "auth": None,
@@ -437,26 +481,35 @@ def test_merge_query_schema(monkeypatch):
 
     routes = Routes(200)
     assert list(routes) == []
-
-    routes.add_route(Context("GET", "/query"))
+    route_metadata = gen_route_metadata(route="/query")
+    routes.initialize_route(route_metadata)
+    routes.increment_route(route_metadata)
     assert list(routes) == [
         {
             "method": "GET",
             "path": "/query",
             "hits": 1,
+            "hits_delta_since_sync": 1,
             "apispec": {},
         },
     ]
+    context1 = Context("GET", "/query", None, query={"test": "abc"})
+    context2 = Context("GET", "/query", None, query={"x": "123", "test": "abc"})
 
-    routes.add_route(Context("GET", "/query", None, query={"test": "abc"}))
-    routes.add_route(Context("GET", "/query", None, query={"x": "123", "test": "abc"}))
-    routes.add_route(Context("GET", "/query"))
+    routes.increment_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context1))
+
+    routes.increment_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context2))
+
+    routes.increment_route(route_metadata)
 
     assert list(routes) == [
         {
             "method": "GET",
             "path": "/query",
             "hits": 4,
+            "hits_delta_since_sync": 4,
             "apispec": {
                 "query": {
                     "type": "object",
@@ -476,16 +529,26 @@ def test_add_auth_schema(monkeypatch):
     monkeypatch.setenv("AIKIDO_FEATURE_COLLECT_API_SCHEMA", "1")
 
     routes = Routes(200)
+    route_metadata1 = gen_route_metadata(route="/auth")
+    context1 = Context("GET", "/auth", headers={"AUTHORIZATION": "Bearer token"})
+    route_metadata2 = gen_route_metadata(route="/auth2")
+    context2 = Context("GET", "/auth2", cookies={"session": "test"})
+    route_metadata3 = gen_route_metadata(route="/auth3")
+    context3 = Context("GET", "/auth3", headers={"X_API_KEY": "token"})
 
-    routes.add_route(Context("GET", "/auth", headers={"AUTHORIZATION": "Bearer token"}))
-    routes.add_route(Context("GET", "/auth2", cookies={"session": "test"}))
-    routes.add_route(Context("GET", "/auth3", headers={"X_API_KEY": "token"}))
+    routes.initialize_route(route_metadata1)
+    routes.update_route_with_apispec(route_metadata1, get_api_info(context1))
+    routes.initialize_route(route_metadata2)
+    routes.update_route_with_apispec(route_metadata2, get_api_info(context2))
+    routes.initialize_route(route_metadata3)
+    routes.update_route_with_apispec(route_metadata3, get_api_info(context3))
 
     assert list(routes) == [
         {
             "method": "GET",
             "path": "/auth",
-            "hits": 1,
+            "hits": 0,
+            "hits_delta_since_sync": 0,
             "apispec": {
                 "body": None,
                 "query": None,
@@ -495,7 +558,8 @@ def test_add_auth_schema(monkeypatch):
         {
             "method": "GET",
             "path": "/auth2",
-            "hits": 1,
+            "hits": 0,
+            "hits_delta_since_sync": 0,
             "apispec": {
                 "body": None,
                 "query": None,
@@ -505,7 +569,8 @@ def test_add_auth_schema(monkeypatch):
         {
             "method": "GET",
             "path": "/auth3",
-            "hits": 1,
+            "hits": 0,
+            "hits_delta_since_sync": 0,
             "apispec": {
                 "body": None,
                 "query": None,
@@ -520,17 +585,29 @@ def test_merge_auth_schema(monkeypatch):
 
     routes = Routes(200)
     assert list(routes) == []
+    route_metadata = gen_route_metadata(route="/auth")
 
-    routes.add_route(Context("GET", "/auth"))
-    routes.add_route(Context("GET", "/auth", headers={"AUTHORIZATION": "Bearer token"}))
-    routes.add_route(Context("GET", "/auth", headers={"AUTHORIZATION": "Basic token"}))
-    routes.add_route(Context("GET", "/auth", headers={"X_API_KEY": "token"}))
-    routes.add_route(Context("GET", "/auth"))
+    routes.initialize_route(route_metadata)
+    routes.increment_route(route_metadata)
+
+    context1 = Context("GET", "/auth", headers={"AUTHORIZATION": "Bearer token"})
+    routes.increment_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context1))
+
+    context2 = Context("GET", "/auth", headers={"AUTHORIZATION": "Basic token"})
+    routes.increment_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context2))
+
+    context3 = Context("GET", "/auth", headers={"X_API_KEY": "token"})
+    routes.increment_route(route_metadata)
+    routes.update_route_with_apispec(route_metadata, get_api_info(context3))
+    routes.increment_route(route_metadata)
 
     assert list(routes)[0] == {
         "method": "GET",
         "path": "/auth",
         "hits": 5,
+        "hits_delta_since_sync": 5,
         "apispec": {
             "auth": [
                 {"type": "http", "scheme": "bearer"},
