@@ -11,17 +11,14 @@ from . import (
 
 def test_aikido_exception_default_message():
     exception = AikidoException()
-    assert (
-        str(exception)
-        == "(AikidoException(...), 'Aikido firewall has blocked unknown')"
-    )
+    assert str(exception) == "(AikidoException(...), 'Zen has blocked unknown')"
 
 
 def test_aikido_sql_injection():
     exception = AikidoSQLInjection(dialect="MySQL")
     assert (
         str(exception)
-        == "(AikidoSQLInjection(...), 'Aikido firewall has blocked an SQL injection, dialect: MySQL')"
+        == "(AikidoSQLInjection(...), 'Zen has blocked an SQL injection, dialect: MySQL')"
     )
 
 
@@ -29,23 +26,20 @@ def test_aikido_nosql_injection():
     exception = AikidoNoSQLInjection()
     assert (
         str(exception)
-        == "(AikidoNoSQLInjection(...), 'Aikido firewall has blocked a NoSQL injection')"
+        == "(AikidoNoSQLInjection(...), 'Zen has blocked a NoSQL injection')"
     )
 
 
 def test_aikido_rate_limiting():
     exception = AikidoRateLimiting()
-    assert (
-        str(exception)
-        == "(AikidoRateLimiting(...), 'You are rate limited by Aikido firewall.')"
-    )
+    assert str(exception) == "(AikidoRateLimiting(...), 'You are rate limited by Zen.')"
 
 
 def test_aikido_shell_injection():
     exception = AikidoShellInjection()
     assert (
         str(exception)
-        == "(AikidoShellInjection(...), 'Aikido firewall has blocked a shell injection')"
+        == "(AikidoShellInjection(...), 'Zen has blocked a shell injection')"
     )
 
 
@@ -53,7 +47,7 @@ def test_aikido_path_traversal():
     exception = AikidoPathTraversal()
     assert (
         str(exception)
-        == "(AikidoPathTraversal(...), 'Aikido firewall has blocked a path traversal attack')"
+        == "(AikidoPathTraversal(...), 'Zen has blocked a path traversal attack')"
     )
 
 
@@ -61,5 +55,5 @@ def test_aikido_ssrf():
     exception = AikidoSSRF()
     assert (
         str(exception)
-        == "(AikidoSSRF(...), 'Aikido firewall has blocked a server-side request forgery')"
+        == "(AikidoSSRF(...), 'Zen has blocked a server-side request forgery')"
     )
