@@ -2,11 +2,17 @@ import pytest
 from unittest.mock import MagicMock, patch
 from . import run_vulnerability_scan
 from aikido_zen.context import current_context, Context
-from aikido_zen.background_process.ipc_lifecycle_cache import (
-    ipc_lifecycle_cache,
-    IPCLifecycleCache,
-)
 from aikido_zen.errors import AikidoSQLInjection
+from aikido_zen.thread.thread_cache import ThreadCache, threadlocal_storage
+
+
+@pytest.fixture(autouse=True)
+def run_around_tests():
+    yield
+    # Make sure to reset context and cache after every test so it does not
+    # interfere with other tests
+    current_context.set(None)
+    setattr(threadlocal_storage, "cache", None)
 
 
 @pytest.fixture
@@ -36,85 +42,81 @@ def get_context():
         body={"test_input_sql": "doggoss2', TRUE"},
         source="flask",
     )
+    context.route_params = {"test_input2": "cattss2', TRUE"}
     return context
 
 
 def test_run_vulnerability_scan_no_context(caplog):
     current_context.set(None)
-    ipc_lifecycle_cache.set(1)
+    threadlocal_storage.cache = 1
     run_vulnerability_scan(kind="test", op="test", args=tuple())
     assert len(caplog.text) == 0
 
 
 def test_run_vulnerability_scan_no_context_no_lifecycle(caplog):
     current_context.set(None)
-    ipc_lifecycle_cache.set(None)
+    threadlocal_storage.cache = None
     run_vulnerability_scan(kind="test", op="test", args=tuple())
     assert len(caplog.text) == 0
 
 
 def test_run_vulnerability_scan_context_no_lifecycle(caplog):
-    current_context.set(1)
-    ipc_lifecycle_cache.set(None)
-    run_vulnerability_scan(kind="test", op="test", args=tuple())
-    assert len(caplog.text) == 0
-
-
-def test_lifecycle_cache_parts(caplog, get_context):
-    get_context.set_as_current_context()
-    cache = IPCLifecycleCache(get_context)
-    cache.matched_endpoints = [{"endpoint": {"forceProtectionOff": True}}]
-    assert cache.protection_forced_off()
-    run_vulnerability_scan(kind="test", op="test", args=tuple())
-    assert len(caplog.text) == 0
+    with pytest.raises(Exception):
+        current_context.set(1)
+        threadlocal_storage.cache = None
+        run_vulnerability_scan(kind="test", op="test", args=tuple())
 
 
 def test_lifecycle_cache_ok(caplog, get_context):
     get_context.set_as_current_context()
-    cache = IPCLifecycleCache(get_context)
-    assert not cache.protection_forced_off()
+    cache = ThreadCache()
     run_vulnerability_scan(kind="test", op="test", args=tuple())
     assert "Vulnerability type test currently has no scans implemented" in caplog.text
 
 
 def test_ssrf(caplog, get_context):
     current_context.set(None)
-    cache = IPCLifecycleCache(get_context)
-    assert not cache.protection_forced_off()
     run_vulnerability_scan(kind="ssrf", op="test", args=tuple())
 
 
 def test_lifecycle_cache_bypassed_ip(caplog, get_context):
     get_context.set_as_current_context()
-    cache = IPCLifecycleCache(get_context)
-    cache.bypassed_ips = {"198.51.100.23"}
+    cache = ThreadCache()
+    cache.config.bypassed_ips = {"198.51.100.23"}
     assert cache.is_bypassed_ip("198.51.100.23")
     run_vulnerability_scan(kind="test", op="test", args=tuple())
     assert len(caplog.text) == 0
 
 
 def test_sql_injection(caplog, get_context, monkeypatch):
-    from aikido_zen.vulnerabilities.sql_injection.dialects import MySQL
-
     get_context.set_as_current_context()
-    cache = IPCLifecycleCache(get_context)
-    assert not cache.protection_forced_off()
-    monkeypatch.setenv("AIKIDO_BLOCKING", "1")
+    cache = ThreadCache()
+    monkeypatch.setenv("AIKIDO_BLOCK", "1")
     with pytest.raises(AikidoSQLInjection):
         run_vulnerability_scan(
             kind="sql_injection",
             op="test_op",
-            args=("INSERT * INTO VALUES ('doggoss2', TRUE);", MySQL()),
+            args=("INSERT * INTO VALUES ('doggoss2', TRUE);", "mysql"),
+        )
+
+
+def test_sql_injection_with_route_params(caplog, get_context, monkeypatch):
+    get_context.set_as_current_context()
+    cache = ThreadCache()
+    monkeypatch.setenv("AIKIDO_BLOCK", "1")
+    with pytest.raises(AikidoSQLInjection):
+        run_vulnerability_scan(
+            kind="sql_injection",
+            op="test_op",
+            args=("INSERT * INTO VALUES ('cattss2', TRUE);", "mysql"),
         )
 
 
 def test_sql_injection_with_comms(caplog, get_context, monkeypatch):
-    from aikido_zen.vulnerabilities.sql_injection.dialects import MySQL
-
     get_context.set_as_current_context()
-    cache = IPCLifecycleCache(get_context)
-    assert not cache.protection_forced_off()
-    monkeypatch.setenv("AIKIDO_BLOCKING", "1")
+    cache = ThreadCache()
+    cache.last_renewal = 9999999999999999999999
+    monkeypatch.setenv("AIKIDO_BLOCK", "1")
     with patch("aikido_zen.background_process.comms.get_comms") as mock_get_comms:
         # Create a mock comms object
         mock_comms = MagicMock()
@@ -123,7 +125,7 @@ def test_sql_injection_with_comms(caplog, get_context, monkeypatch):
             run_vulnerability_scan(
                 kind="sql_injection",
                 op="test_op",
-                args=("INSERT * INTO VALUES ('doggoss2', TRUE);", MySQL()),
+                args=("INSERT * INTO VALUES ('doggoss2', TRUE);", "mysql"),
             )
         mock_comms.send_data_to_bg_process.assert_called_once()
         call_args = mock_comms.send_data_to_bg_process.call_args[0]
@@ -136,12 +138,8 @@ def test_sql_injection_with_comms(caplog, get_context, monkeypatch):
 
 
 def test_ssrf_with_comms_hostnames_add(caplog, get_context, monkeypatch):
-    from aikido_zen.vulnerabilities.sql_injection.dialects import MySQL
-
     get_context.set_as_current_context()
-    cache = IPCLifecycleCache(get_context)
-    assert not cache.protection_forced_off()
-    monkeypatch.setenv("AIKIDO_BLOCKING", "1")
+    monkeypatch.setenv("AIKIDO_BLOCK", "1")
     with patch("aikido_zen.background_process.comms.get_comms") as mock_get_comms:
         # Create a mock comms object
         mock_comms = MagicMock()
@@ -151,7 +149,6 @@ def test_ssrf_with_comms_hostnames_add(caplog, get_context, monkeypatch):
             op="test_op",
             args=([], "test-hostname", 8097),
         )
-        mock_comms.send_data_to_bg_process.assert_called_once()
-        call_args = mock_comms.send_data_to_bg_process.call_args[0]
-        assert call_args[0] == "HOSTNAMES_ADD"
-        assert call_args[1] == ("test-hostname", 8097)
+        mock_comms.send_data_to_bg_process.assert_any_call(
+            "HOSTNAMES_ADD", ("test-hostname", 8097)
+        )
